unit dOPCHDADlgBrowse;
interface

{$IFDEF VER150}
{$WARN UNSAFE_TYPE OFF}
{$ENDIF}


uses
  Classes, Controls, Forms, Buttons, ComCtrls, StdCtrls, dOPCHDA, ImgList,
  Menus;

type
  TdOPCDlgHDABrowse = class(TForm)
    ListView: TListView;
    bBack: TSpeedButton;
    bOk: TButton;
    bCancel: TButton;
    Label3: TLabel;
    ePath: TEdit;
    ImageList1: TImageList;
    bView: TSpeedButton;
    PopupMenu1: TPopupMenu;
    Detail1: TMenuItem;
    List1: TMenuItem;
    Reporrt1: TMenuItem;
    Icon1: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure bBackClick(Sender: TObject);
    procedure ListViewDblClick(Sender: TObject);
    procedure UpdateEvent(Sender: TObject);
    procedure bOkClick(Sender: TObject);
    procedure ListViewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure bViewClick(Sender: TObject);
    procedure Detail1Click(Sender: TObject);
    procedure List1Click(Sender: TObject);
    procedure Reporrt1Click(Sender: TObject);
    procedure Icon1Click(Sender: TObject);
  private
     ToRoot  : boolean;
     Browser : TdOPCHDABrowser;
     ItemList: TStrings;
     procedure ShowPrevLevel;
     function  ShowNextLevel: boolean;
     procedure ShowLevel;
     procedure GetItemList;
     procedure ClearListView;
  public
  end;


// Opendialog to select Items from OPC Server
// @param Browser OPCServer Browser
// @param Itemlist list of selected Items if result is true
// @param ToRoot  if false the actual Browserposition is used
//                if true moves to the Root
// result true  = user has select Items see also ItemList
//        false = user has canceled Dialog
function dOPCHDABrowseDlg(Browser: TdOPCHDABrowser;
                         ItemList: TStrings;
                         ToRoot: boolean = false): boolean;

// the same as OPCBrowseDialog without multiselect
// result =  '' if cancel else ItemId
function dOPCHDABrowseItemDlg(Browser: TdOPCHDABrowser;ToRoot: boolean = false): string;


implementation
uses
  Windows, Messages, dOPCIntf;

{$R *.DFM}

function dOPCHDABrowseDlg(Browser: TdOPCHDABrowser;
                         ItemList: TStrings;
                         ToRoot: boolean = false): boolean;
var
  D : TdOPCDlgHDABrowse;
begin
 result := false;
 if Browser.OPCServer = nil then exit;
 if Browser.OPCServer.Active = false then exit;
 Application.CreateForm(TdOPCDlgHDABrowse,D);
 D.Browser  := Browser;
 D.ItemList := ItemList;
 D.ToRoot   := ToRoot;
 if (D.ShowModal() = idOk) then
   result := true;
 D.free();
end;


function dOPCHDABrowseItemDlg(Browser: TdOPCHDABrowser;ToRoot: boolean = false): string;
var
  D : TdOPCDlgHDABrowse;
 ItemList : TStringList;
begin
 result := '';
 if Browser.OPCServer = nil then exit;
 if Browser.OPCServer.Active = false then exit;
 ItemList := TStringList.create;
 Application.CreateForm(TdOPCDlgHDABrowse,D);
 D.Browser  := Browser;
 D.ItemList := ItemList;
 D.ToRoot   := ToRoot;
 D.ListView.MultiSelect := false;
 if (D.ShowModal() = idOk) then
   result := ItemList[0];;
 D.free();
 ItemList.Free;
end;



procedure TdOPCDlgHDABrowse.FormShow(Sender: TObject);
begin
  if ToRoot then
    Browser.MovetoRoot;
  ShowLevel;                 // show branches and leafs in listview
end;

// clear all Item in Listview and Browseritems
procedure TdOPCDlgHDABrowse.ClearListView;
begin
  Listview.Items.Clear;
end;

// shows all branches and Leafs in the actual Level
procedure TdOPCDlgHDABrowse.ShowLevel;
var
  Item : TListItem;
  i    : integer;
  BrowseItem : TdOPCBrowseItem;
begin
   // clear all Item in Listview
   ClearListView;
   if Browser.CurrentPosition.Name = '' then
     EPath.Text := 'root'
   else
     EPath.Text := Browser.CurrentPosition.Name;
   // add branches to Listview
   Browser.Browse;
   for i := 0 to Browser.Items.Count-1 do
   begin
     Item := Listview.Items.Add;
     BrowseItem := Browser.Items[i];
     Item.Caption := BrowseItem.Name;
     Item.Data    := BrowseItem;
     if BrowseItem.IsFolder then
        Item.ImageIndex := 0             // branch icon
     else
        Item.ImageIndex := 1;            // Leaf icon
   end;

   if Listview.Items.Count > 0 then
   begin
     Listview.Selected := Listview.Items[0];  // select first Item
     Listview.Selected.Focused := true;
   end;
end;


procedure TdOPCDlgHDABrowse.ShowPrevLevel;
begin
  if Browser.CurrentPosition.ItemName <> '' then
  begin
    Browser.Moveup;
    ShowLevel;
  end;
end;

function TdOPCDlgHDABrowse.ShowNextLevel: boolean;
var
  Item : TListItem;
begin
   result := false;
   Item := ListView.Selected;
   if Item <> nil then
   begin
     if Item.ImageIndex = 0 then   // if branch
     begin
       Browser.MoveDown(Item.Data);
       ShowLevel;
       result := true;
     end
   end;
end;

procedure TdOPCDlgHDABrowse.bBackClick(Sender: TObject);
begin
  ShowPrevLevel;
end;

procedure TdOPCDlgHDABrowse.ListViewDblClick(Sender: TObject);
begin
  bOkClick(self);
end;

procedure TdOPCDlgHDABrowse.ListViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_SPACE then
    ShowNextLevel;
  if key = VK_BACK then
    ShowPrevLevel;
end;

// is called if Filter or Datatype or Read or write is changed
procedure TdOPCDlgHDABrowse.UpdateEvent(Sender: TObject);
begin
  ShowLevel;
end;

// write all selected leafs in Itemlist
procedure TdOPCDlgHDABrowse.GetItemList;
var
  Item : TListItem;
  i    : integer;
  BrowseItem : TdOPCBrowseItem;
begin
   ItemList.Clear;
   for i := 0 to Listview.Items.Count -1 do             // for all items in view
   begin
      Item := Listview.Items[i];                        // get item
      if (Item.Imageindex = 1) and Item.Selected then   // if selected an leaf
      begin
         BrowseItem := TdOPCBrowseItem(Item.Data);
         ItemList.Add(BrowseItem.ItemId); // result = ItemIds
      end;
   end;
end;

procedure TdOPCDlgHDABrowse.bOkClick(Sender: TObject);
begin
  ItemList.Clear;
  if not ShowNextLevel then  // if leaf is selected
    GetItemList;
  if ItemList.Count > 0 then
  begin
    Close;
    Modalresult := mrOk;
  end;
end;


procedure TdOPCDlgHDABrowse.bViewClick(Sender: TObject);
var
  P : TPoint;
begin
    P := bView.ClientOrigin;
    P.Y := P.Y+bView.Height;
    PopupMenu1.Popup(P.x,P.y);
end;

procedure TdOPCDlgHDABrowse.Detail1Click(Sender: TObject);
begin
  ListView.ViewStyle := vsReport;
end;

procedure TdOPCDlgHDABrowse.List1Click(Sender: TObject);
begin
  ListView.ViewStyle := vsList;
end;

procedure TdOPCDlgHDABrowse.Reporrt1Click(Sender: TObject);
begin
 ListView.ViewStyle := vsSmallIcon;
end;

procedure TdOPCDlgHDABrowse.Icon1Click(Sender: TObject);
begin
 ListView.ViewStyle := vsIcon;

end;

end.


